/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.topology;

import java.util.ArrayList;
import java.util.Collection;

import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.Topology;

/**
 * @author Adrien Louis - EBM WebSourcing
 */
public final class TopologyHelper {

    /**
     * No constructor available.
     */
    private TopologyHelper() {
        // NOP
    }

    /**
     * Retrieve the sub domain in the given topology
     *
     * @param subdomainName
     *            may be null
     * @param topology
     *            must be non null
     * @return the sub-domain, null if not found
     */
    public static Subdomain findSubdomain(final String subdomainName,
            final Topology topology) {
        Subdomain result = null;
        for (Subdomain subdomain : topology.getDomain().getSubDomain()) {
            if (subdomain.getName().equals(subdomainName)) {
                result = subdomain;
                break;
            }
        }
        return result;
    }

    /**
     * <p>
     * Retrieve the sub domain in the given topology from the given container.
     * </p>
     * <p>
     * Only the attribute 'name' must be defined for the container.
     * </p>
     * 
     * @param container
     *            must be non null
     * @param topology
     *            must be non null
     * @return the sub-domain, null if not found
     */
    public static Subdomain findSubdomain(final Container container,
            final Topology topology) {
        Subdomain result = null;
        for (final Subdomain subdomain : topology.getDomain().getSubDomain()) {
            if (TopologyHelper.findContainer(container.getName(), subdomain) != null) {
                result = subdomain;
                break;
            }
        }
        return result;
    }

    /**
     * Retrieve the container in the given topology
     *
     * @param containerName
     *            may be null
     * @param topology
     *            must be non null
     * @return the container, null if not found
     */
    public static Container findContainer(final String containerName,
            final Topology topology) {
        Container result = null;
        for (Subdomain subdomain : topology.getDomain().getSubDomain()) {
            result = TopologyHelper.findContainer(containerName, subdomain);
            if (result != null) {
                break;
            }
        }
        return result;
    }

    /**
     * Retrieve the container in the given Subdomain
     *
     * @param containerName
     *            may be null
     * @param subdomain
     *            must be non null
     * @return the container, null if not found
     */
    public static Container findContainer(final String containerName,
            final Subdomain subdomain) {
        Container result = null;
        for (Container container : subdomain.getContainer()) {
            if (container.getName().equals(containerName)) {
                result = container;
                break;
            }
        }
        return result;
    }

    /**
     * Retrieve all the containers in the given topology
     *
     * @param topology
     *            must be non null
     * @return collection of all the containers of the topology, not null, can be empty
     */
    public static Collection<Container> findAllContainers(final Topology topology) {
        final Collection<Container> result = new ArrayList<Container>();
        for (Subdomain subdomain : topology.getDomain().getSubDomain()) {
            for (Container container : subdomain.getContainer()) {
                result.add(container);
            }
        }
        return result;
    }

}
